-------------------------------------------------------------------------------------
-- VideoCAD.ms
-- Exports entire scenes or selected objects to VideoCAD model (*.vcm) format
-- By Stanislav Utochkin 
-- Based on MEGA script created by Ivan Dishlenko
-- Version 1.0 
-------------------------------------------------------------------------------------
utility MEGExport "VideoCAD"
(
	local ostream, tabs = ""

	group "Model ID"
	(
        spinner model_id "ID" range:[-512000,512000,100] type:#Integer
	)
	group "Options"
	(
		checkbox cb_exportSelOnly "Selected Only"
    )
    group "License plates"
	(
		checkbox cb_numberFront "At front"
		edittext number_Front_X "X:"
        edittext number_Front_Y "Y:"
        edittext number_Front_Z "Z:"
		
		checkbox cb_numberBack "At back"
		edittext number_Back_X "X:"
        edittext number_Back_Y "Y:"
        edittext number_Back_Z "Z:"				
	)

	button btn_export "Save As..." width:100
	button btn_about "About" width:100

	-------------------------------------------------------------------------------------
		function ExportTarget obj =
	(
		Format (tabs+"myobj.target = Targetobject()\n") to:ostream
		Format (tabs+"myobj.target.name = \"%\"\n") obj.target.name to:ostream
		Format (tabs+"myobj.target.transform = %\n") obj.target.transform to:ostream
		Format (tabs+"myobj.target.position += createPos\n") to:ostream
	)

        -------------------------------------------------------------------------------------
        function ExportGeneric obj =
	(
		Format (tabs+"%\n") (ClassOf obj) to:ostream

		for prop in GetPropNames obj do
		(
			local propname = prop as String
			local propval  = GetProperty obj prop

			if (ClassOf obj) == Hedra and prop == #vertices then continue

			if propval != undefined then
				Format (tabs+"\t% %\n") propname propval to:ostream
		)
		
		if obj.target != undefined then
			ExportTarget obj

		Format (tabs+"end %\n") (ClassOf obj) to:ostream
	)

	-------------------------------------------------------------------------------------
	function ExportMesh meshObj =
	(
		Format (tabs+"TriMesh()\n") to:ostream
		Format (tabs+"numverts numfaces\n") to:ostream
		Format (tabs+"   %        %\n") meshObj.numverts meshObj.numfaces to:ostream
		Format (tabs+"%\n") (meshObj.wirecolor.r/255) to:ostream
		Format (tabs+"%\n") (meshObj.wirecolor.g/255) to:ostream
		Format (tabs+"%\n") (meshObj.wirecolor.b/255) to:ostream

		-- Write vertices
		if meshObj.numVerts > 0 then
		(

			Format (tabs+"Mesh vertices:\n") to:ostream
			for i = 1 to meshObj.numVerts do
			(
				Format (tabs+"\t% % %\n") ((GetVert meshObj i).x) ((GetVert meshObj i).y) ((GetVert meshObj i).z) to:ostream 	
			)
			Format (tabs+"end vertices\n") to:ostream
		)


		-- Write faces
		if meshObj.numFaces > 0 then
		(
			Format (tabs+"Mesh faces:\n") to:ostream
			for i = 1 to meshObj.numFaces do
                        (
				Format (tabs+"\t% % %\n") ((GetFace meshObj i).x as Integer) ((GetFace meshObj i).y as integer) ((GetFace meshObj i).z as Integer) to:ostream
                        )
			Format (tabs+"end faces\n") to:ostream
		)

		-- Write smoothing normals
		if meshObj.numVerts > 0 then
		(
			Format (tabs+"Smooth normals:\n") to:ostream
			for i = 1 to meshObj.numVerts do
			(
				Format (tabs+"\t% % %\n") (GetNormal meshObj i).x (GetNormal meshObj i).y (GetNormal meshObj i).z to:ostream 	
			)
			Format (tabs+"end smooth normals\n") to:ostream
		)
		
		Format (tabs+"end mesh\n") to:ostream
	)

	-------------------------------------------------------------------------------------
	function ExportNode node =
	(
		Format "New object\n" to:ostream
		if SuperClassOf node == GeometryClass and ClassOf node == Editable_mesh then
			ExportMesh node
		else if (not (ClassOf node == Targetobject and node.isTarget)) then
			ExportGeneric node
		else
			return false

		return true
	)

	-------------------------------------------------------------------------------------
	function RecursiveExportNode node =
	(
		if (ExportNode node) == false then
			return false

		-- Recurse children before writing this node
		for child in node.children do
			RecursiveExportNode child
			
		Format "\n\n" to:ostream
	)

	-------------------------------------------------------------------------------------
	function ExportGMS =
	(
		if cb_exportSelOnly.checked then
		(
			for node in selection do
				ExportNode node
			
		)
		else
		(
			for node in rootnode.children do
				RecursiveExportNode node
		)
	)

	-------------------------------------------------------------------------------------
	function GetSaveFileStream =
	(
		fname = GetSaveFileName types:"VideoCAD Models Files (*.vcm)|*.vcm|All Files(*.*)|*.*|"
		if fname == undefined then
			return undefined

		ostream = CreateFile fname
		if ostream == undefined then
		(
			MessageBox "Couldn't open file for writing !"
			return undefined
		)

		return ostream
	)

	-------------------------------------------------------------------------------------
	on btn_about pressed do
	(
		MessageBox " Meshes Export in VideoCAD model format v1.0\n Copyright: Stanislav Utochkin 2005\n Based on MEGA script.  Copyright: Ivan Dishlenko 2001" title:"About VideoCAD"
	
	) -------------------------------------------------------------------------------------
	on btn_export pressed do
	(
		ostream = GetSaveFileStream()
		if ostream != undefined then
		(
	Format (tabs+"%\n") model_id.value to:ostream
	
			if cb_numberFront.checked then
			Format (tabs+"%\n") 1 to:ostream
			else
			Format (tabs+"%\n") 0 to:ostream
			
			if number_Front_X.text != "" then
			Format (tabs+"%\n") ((number_Front_X.text as Float)*0.01) to:ostream
			else
			Format (tabs+"%\n") 0 to:ostream
			
			if number_Front_Z.text != "" then
			Format (tabs+"%\n") ((number_Front_Z.text as Float)*0.01) to:ostream
			else
			Format (tabs+"%\n") 0 to:ostream			
			
			if number_Front_Y.text != "" then
			Format (tabs+"%\n") (-(number_Front_Y.text as Float)*0.01) to:ostream
			else
			Format (tabs+"%\n") 0 to:ostream

			
			if cb_numberBack.checked then
			Format (tabs+"%\n") 1 to:ostream
			else
			Format (tabs+"%\n") 0 to:ostream
			
			if number_Back_X.text != "" then
			Format (tabs+"%\n") ((number_Back_X.text as Float)*0.01) to:ostream
			else
			Format (tabs+"%\n") 0 to:ostream
			
			if number_Back_Z.text != "" then
			Format (tabs+"%\n") ((number_Back_Z.text as Float)*0.01) to:ostream
			else
			Format (tabs+"%\n") 0 to:ostream			
			
			if number_Back_Y.text != "" then
			Format (tabs+"%\n") (-(number_Back_Y.text as Float)*0.01) to:ostream
			else
			Format (tabs+"%\n") 0 to:ostream
			
			
			
			Format (tabs+"%\n") "" to:ostream
			
			i = 1
			(
				slidertime = 1
				ExportGMS()
		    )
			
		        Format "end of file" to:ostream 
			Close ostream
		)
	)
	--------------------------------------------------------------------------------------

) -- End MEGExport


